/*
     Copyright (C) 1996 - 1997  Masahito Yamaga (yamaga@ipc.chiba-u.ac.jp)

     2002/ 2/10 Nobuyuki Tsuchimura (tutimura@nn.iij4u.or.jp)
     -- use kpse_find_file()
     -- close file
*/

#include "xdvi-config.h"
#include "xdvi.h"
#include "dvi-init.h"
#include "kpathsea/tex-file.h"
#ifndef	DEFAULT_FONTMAP
#define	DEFAULT_FONTMAP	"/usr/local/share/texmf/xdvi/vfontmap"
#endif
#ifdef VF2FT
#include "vf2ft.h"
#endif
#include <ctype.h>

#define NUM(a)	  ((int)(sizeof(a)/sizeof(*(a))))

struct vfontmap *first_ptr = NULL;

static struct jfont_list_st {
    char *jfmname;
    char *vfname;
    int used;
} default_jfonts[] = {
#ifdef PTEX
    { "jisg" ,"goth", False },
    { "jis"  ,"min" , False },
#endif
    { "dg"   ,"goth", False },
    { "dm"   ,"min" , False },

    { "ngoth","goth", False },
    { "tgoth","goth", False },
    { "goth" ,"goth", False },

    { "nmin" ,"min" , False },
    { "tmin" ,"min" , False },
    { "min"  ,"min" , False },
};
    

void addfontmap(char *s1, char *s2, int tate, int encoding, int cidv)
{
    struct vfontmap *new_ptr;

    new_ptr = xmalloc(sizeof(struct vfontmap));
    new_ptr->jfmname = xstrdup(s1);
    new_ptr->vfname  = xstrdup(s2);
#ifdef VF2FT
    new_ptr->encoding = encoding;
    new_ptr->cidv = cidv;
#endif
    new_ptr->flag = -1;
#ifdef PTEX
    new_ptr->tate = tate;
#endif

    new_ptr->next_ptr = first_ptr;
    first_ptr = new_ptr;
}


#if defined(USE_ZEIT) && defined(USE_VFONTMAP)
const char *get_vfontmap_name(void)
{
    char *s;
    static char *vfontmap = "vfontmap";
    static char *default_fontmap = DEFAULT_FONTMAP;
    FILE *fp;

    if ((fp=fopen(vfontmap, "r")) != NULL) {
	fclose(fp);
	return vfontmap;
    }

    if (strcmp(resource.vfontmap, default_fontmap) != 0) {
	return resource.vfontmap;
    }
    s = getenv("VFONTMAP");
    if (s != NULL) return s;
#if 0 /* ADHOC FIX */
    s = kpse_find_file(vfontmap, kpse_program_text_format, 1);
    if (s != NULL) return s;
#endif
    return default_fontmap;
}
#endif /* USE_ZEIT && USE_VFONTMAP */


void read_VFontmap(void)
{
    FILE *map;
    const char *vfontmapf;
    char *s, *jfmname, *vfname, *encode;
    char line[BUFSIZ], line_orig[BUFSIZ], vfname2[BUFSIZ] = "";
    int i, tate, encoding, cidv;
#ifdef VF2FT
    static char *unicodekeystr = "unicode";
    static char *adobejapankeystr = "adobejapan";
    static char *cidvkeystr = "cidv";
#endif

#if defined(USE_ZEIT) && defined(USE_VFONTMAP)
    vfontmapf = get_vfontmap_name();
    if ((map = fopen(vfontmapf, "r")) == NULL) {
	fprintf(stderr,"Warning: Can't open \"%s\"\n", vfontmapf);
	fprintf(stderr,"         Default Values are Used.\n");
    } else {
	while (fgets(line, BUFSIZ, map) != NULL) {
	    if ((s=strchr(line,'\n')) != NULL) *s = '\0';  /* break \n */
	    strcpy(line_orig, line);
	    if ((s=strchr(line,'%')) != NULL) *s = '\0';  /* break comment */
	    if ((s=strchr(line,'#')) != NULL) *s = '\0';  /* break comment */
	    tate = False;  encoding = 0; cidv = False;

	    if (*line == '/') {  // font file for vf2ft
		strcpy(vfname2, strtok(line, " \t"));
		continue;
	    }
	    if (isspace(*line)) {
		for (s=line; isspace(*s); s++) /**/;
		jfmname = strtok(s, " \t");
		vfname = vfname2;
	    } else {
		jfmname = strtok(line, " \t");
		vfname = strtok(NULL, " \t");
	    }
#ifdef PTEX
	    encode = strtok(NULL, " \t");
#ifdef VF2FT
	    if (encode != NULL) {
		if (strcmp(encode, unicodekeystr) == 0) {
		    encoding = UNIC;
		} else if (strcmp(encode, adobejapankeystr) == 0) {
		    encoding = ADOBEJAPAN;
		} else if (strcmp(encode, cidvkeystr) == 0) {
		    cidv = True;
		    tate = True;
		} else {
		    tate = True;
		}
	    }
#else /* !VF2FT */
	    tate = (encode != NULL);
#endif /* !VF2FT */
#endif /* PTEX */
	    if (jfmname == NULL) continue;
	    if (vfname == NULL) {
		fprintf(stderr,
			"No specified Vector Font name for \"%s\"\n", jfmname);
		continue;
	    }

	    if (strtok(NULL, " \n\r") != NULL) {
		fprintf(stderr,
			"'%s', There is too much argument.\n", line_orig);
		continue;
	    }

	    if (globals.debug & DBG_PK) {
		fprintf(stderr, "JFM name is \"%s\"\n", jfmname);
		fprintf(stderr, "Font name of VFlib is \"%s\"\n", vfname);
#ifdef PTEX
#ifdef VF2FT
		fprintf(stderr, "Font of FreeType is %s encode \"%s\"\n",
			(encoding != 0) ? encode : "default",
			tate ? "TATE font" : "TRANS...font");
#else /* !VF2FT */
		fprintf(stderr, "Font of VFlib is \"%s\"\n",
			tate ? "TATE font" : "TRANS...font");
#endif /* !VF2FT */
#endif
	    }
	    addfontmap(jfmname, vfname, tate, encoding, cidv);

	    /* check if necessary fontmap? */
	    for (i=0; i<NUM(default_jfonts); i++) {
		if (strcmp(jfmname, default_jfonts[i].jfmname) == 0) {
		    default_jfonts[i].used = True;
		}
	    }
	}
	fclose(map);
    }
#endif /* USE_ZEIT && USE_VFONTMAP */

    /* add necessary fontmaps */
    for (i=0; i<NUM(default_jfonts); i++) {
	if (!default_jfonts[i].used) {
	    addfontmap(default_jfonts[i].jfmname, 
		       default_jfonts[i].vfname, 0, 0, 0);
	}
    }
}
